/*! For license information please see 35.bc08e7dc.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[35],{5767:function(e,t,r){"use strict";r.r(t),r.d(t,"default",(function(){return n}));r(0);var i=r(30),s=r(5);var n=function(e){var t=e.disabled,r=e.element,n=e.widgetMgr,l={width:e.width};return Object(s.jsx)("div",{className:"row-widget stButton",style:l,children:Object(s.jsx)(i.a,{help:r.help,children:Object(s.jsx)(i.d,{kind:i.b.PRIMARY,size:i.c.SMALL,disabled:t,onClick:function(){return n.setTriggerValue(r,{fromUi:!0})},children:r.label})})})}}}]);