/*! For license information please see 34.ecc8432c.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[34],{5766:function(e,t,r){"use strict";r.r(t),r.d(t,"default",(function(){return s}));var n=r(0),a=r(21),c=r(72),i=r(5);function s(e){var t=e.element,r=e.width,s=Object(n.useRef)(null),o=t.type,u=t.url;Object(n.useEffect)((function(){var e=s.current,r=function(){e&&(e.currentTime=t.startTime)};return e&&e.addEventListener("loadedmetadata",r),function(){e&&e.removeEventListener("loadedmetadata",r)}}),[t]);if(o===a.u.Type.YOUTUBE_IFRAME){var l=.75*r;return Object(i.jsx)("iframe",{title:u,src:function(e){var r=t.startTime;return r?"".concat(e,"?start=").concat(r):e}(u),width:r,height:l,frameBorder:"0",allow:"autoplay; encrypted-media",allowFullScreen:!0})}return Object(i.jsx)("video",{ref:s,controls:!0,src:Object(c.b)(u),className:"stVideo",style:{width:r}})}}}]);