# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/RootContainer.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='streamlit/proto/RootContainer.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n#streamlit/proto/RootContainer.proto*&\n\rRootContainer\x12\x08\n\x04MAIN\x10\x00\x12\x0b\n\x07SIDEBAR\x10\x01\x62\x06proto3'
)

_ROOTCONTAINER = _descriptor.EnumDescriptor(
  name='RootContainer',
  full_name='RootContainer',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MAIN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SIDEBAR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=39,
  serialized_end=77,
)
_sym_db.RegisterEnumDescriptor(_ROOTCONTAINER)

RootContainer = enum_type_wrapper.EnumTypeWrapper(_ROOTCONTAINER)
MAIN = 0
SIDEBAR = 1


DESCRIPTOR.enum_types_by_name['RootContainer'] = _ROOTCONTAINER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


# @@protoc_insertion_point(module_scope)
