/*! For license information please see 22.c6a2a1a4.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[22],{4346:function(t,i){},5756:function(t,i,e){"use strict";e.r(i),e.d(i,"default",(function(){return m}));e(0);var r,n=e(4322),a=e.n(n),o=e(213),c=e(72),d=e(13),l=e.n(d),s=l()("div",{target:"etr89bj0"})((function(t){return{display:"flex",flexDirection:"row",flexWrap:"wrap",rowGap:t.theme.spacing.lg}}),""),h=l()("div",{target:"etr89bj1"})((function(t){t.theme;return{display:"flex",flexDirection:"column",alignItems:"stretch",width:"auto",flexGrow:0}}),""),u=l()("div",{target:"etr89bj2"})((function(t){var i=t.theme;return{fontFamily:i.genericFonts.bodyFont,fontSize:i.fontSizes.sm,color:i.colors.fadedText60,textAlign:"center",marginTop:i.spacing.xs,wordWrap:"break-word",padding:"0.125rem"}}),""),f=e(5);!function(t){t[t.OriginalWidth=-1]="OriginalWidth",t[t.ColumnWidth=-2]="ColumnWidth",t[t.AutoWidth=-3]="AutoWidth"}(r||(r={}));var m=Object(o.a)((function(t){var i,e=t.width,n=t.isFullScreen,o=t.element,d=t.height,l=o.width;if(l===r.OriginalWidth||l===r.AutoWidth)i=void 0;else if(l===r.ColumnWidth)i=e;else{if(!(l>0))throw Error("Invalid image width: ".concat(l));i=l}var m={};return d&&n?(m.maxHeight=d,m["object-fit"]="contain"):(m.width=i,l===r.AutoWidth&&(m.maxWidth="100%")),Object(f.jsx)(s,{style:{width:e},children:o.imgs.map((function(t,i){var e=t;return Object(f.jsxs)(h,{"data-testid":"stImage",children:[e.markup?a()(Object(c.f)(e.markup)):Object(f.jsx)("img",{style:m,src:Object(c.b)(e.url),alt:i.toString()}),e.caption&&Object(f.jsx)(u,{"data-testid":"caption",style:m,children:" ".concat(e.caption," ")})]},i)}))})}))}}]);