/*! For license information please see 32.70c1b2f7.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[32],{5758:function(e,t,n){"use strict";n.r(t),n.d(t,"default",(function(){return g}));n(0);var r=n(37),o=n(104),i=n(13),s=n.n(i),a=n(271),c=n(254),l=n.n(c),d=s()(a.b,{shouldForwardProp:l.a,target:"e17lx80j0"})((function(e){var t=e.theme;return{marginTop:t.spacing.none,marginBottom:t.spacing.none,marginRight:t.spacing.none,marginLeft:t.spacing.none,borderColor:t.colors.fadedText10,borderTopColor:t.colors.blue,flexGrow:0,flexShrink:0}}),""),p=s()("div",{target:"e17lx80j1"})((function(e){return{display:"flex",gap:e.theme.spacing.lg,alignItems:"center",width:"100%"}}),""),u=n(5);var g=function(e){var t=e.width,n=e.element,i=Object(r.useTheme)(),s={width:t};return Object(u.jsx)("div",{className:"stSpinner",style:s,children:Object(u.jsxs)(p,{children:[Object(u.jsx)(d,{$size:i.iconSizes.twoXL}),Object(u.jsx)(o.a,{source:n.text,allowHTML:!1})]})})}}}]);