/*! For license information please see 39.60537b6f.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[39],{5768:function(e,t,r){"use strict";r.r(t),r.d(t,"default",(function(){return l}));r(0);var i=r(30),n=r(72),c=r(5);var l=function(e){var t=e.disabled,r=e.element,l=e.widgetMgr,s={width:e.width};return Object(c.jsx)("div",{className:"row-widget stDownloadButton",style:s,children:Object(c.jsx)(i.a,{help:r.help,children:Object(c.jsx)(i.d,{kind:i.b.PRIMARY,size:i.c.SMALL,disabled:t,onClick:function(){l.setTriggerValue(r,{fromUi:!0});var e=document.createElement("a"),t="".concat(Object(n.b)(r.url),"?title=").concat(encodeURIComponent(document.title));e.setAttribute("href",t),e.setAttribute("target","_blank"),e.click()},children:r.label})})})}}}]);