/*! For license information please see 27.edcfaef8.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[27],{5762:function(t,e,r){"use strict";r.r(e),r.d(e,"default",(function(){return i}));var s=r(0),u=r(72),c=r(5);function i(t){var e=t.element,r=t.width,i=Object(s.useRef)(null);Object(s.useEffect)((function(){i.current&&(i.current.currentTime=e.startTime)}),[e.startTime]);var n=Object(u.b)(e.url);return Object(c.jsx)("audio",{id:"audio",ref:i,controls:!0,src:n,className:"stAudio",style:{width:r}})}}}]);