/*! For license information please see 30.bc29a6fc.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[30],{5764:function(t,r,e){"use strict";e.r(r),e.d(r,"default",(function(){return n}));var i=e(350),s=(e(0),e(5));function n(t){var r,e,n=t.element,l=t.width,c=n.hasWidth?n.width:l;n.scrolling?(r="auto",e={}):(r="no",e={overflow:"hidden"});var a=o(n.src),h=null!=a?void 0:o(n.srcdoc);return Object(s.jsx)("iframe",{allow:i.a,style:e,src:a,srcDoc:h,width:c,height:n.height,scrolling:r,sandbox:i.b,title:"st.iframe"})}function o(t){return null==t||""===t?void 0:t}}}]);