(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[41],{5773:function(e,t,o){"use strict";o.r(t),o.d(t,"default",(function(){return c}));var n=o(6),r=o(9),a=o(7),i=o(8),u=o(0),l=o.n(u),s=o(211),p=o(160),m=o(5),c=function(e){Object(a.a)(o,e);var t=Object(i.a)(o);function o(){var e;Object(n.a)(this,o);for(var r=arguments.length,a=new Array(r),i=0;i<r;i++)a[i]=arguments[i];return(e=t.call.apply(t,[this].concat(a))).formClearHelper=new s.b,e.state={value:e.initialValue},e.commitWidgetValue=function(t){e.props.widgetMgr.setIntValue(e.props.element,e.state.value,t)},e.onFormCleared=function(){e.setState({value:e.props.element.default},(function(){return e.commitWidgetValue({fromUi:!0})}))},e.onChange=function(t){e.setState({value:t},(function(){return e.commitWidgetValue({fromUi:!0})}))},e.render=function(){var t=e.props.element,o=t.options,n=t.help,r=t.label,a=t.formId,i=e.props,u=i.disabled,l=i.widgetMgr;return e.formClearHelper.manageFormClearListener(l,a,e.onFormCleared),Object(m.jsx)(p.b,{label:r,options:o,disabled:u,width:e.props.width,onChange:e.onChange,value:e.state.value,help:n})},e}return Object(r.a)(o,[{key:"initialValue",get:function(){var e=this.props.widgetMgr.getIntValue(this.props.element);return void 0!==e?e:this.props.element.default}},{key:"componentDidMount",value:function(){this.props.element.setValue?this.updateFromProtobuf():this.commitWidgetValue({fromUi:!1})}},{key:"componentDidUpdate",value:function(){this.maybeUpdateFromProtobuf()}},{key:"componentWillUnmount",value:function(){this.formClearHelper.disconnect()}},{key:"maybeUpdateFromProtobuf",value:function(){this.props.element.setValue&&this.updateFromProtobuf()}},{key:"updateFromProtobuf",value:function(){var e=this,t=this.props.element.value;this.props.element.setValue=!1,this.setState({value:t},(function(){e.commitWidgetValue({fromUi:!1})}))}}]),o}(l.a.PureComponent)}}]);